import traceback
from walabot_handler import WalabotHandler
from walabot_mqtt_publisher import WalabotMQTT
import RPi.GPIO as GPIO
from sys import platform
from os import system
from imp import load_source
from enum import Enum
import bluetooth
import time
import commands

# Setup GPIO for the LED (connected on GPIO pin 19) 
GPIO.setmode(GPIO.BCM)
GPIO.setwarnings(False)
GPIO.setup(19,GPIO.OUT)
GPIO.output(19,GPIO.LOW)
#global counter for the number of detected persons
num_of_people = 0

arena_params = {
               'min_in_cm': 5,       'max_in_cm': 150,     'res_in_cm': 5,
               'min_in_degrees': -20,  'max_in_degrees': 20,  'res_in_degrees': 10,
               'min_phi_degrees': -60, 'max_phi_degrees': 60, 'res_phi_degrees': 3
                }

def PrintTrackerTargets(targets):
    system('cls' if platform == 'win32' else 'clear')
    if targets:
        for i, target in enumerate(targets):
            print(('y: {}'.format(target.yPosCm)))
    else:
        print('No Target Detected')

class Placement(Enum):
    Empty = 0  # Target not in the arena
    Back  = 1  # Target in the back of the arena
    Front = 2  # Target in the front of the arena

class State(Enum):
    Idle = 0  # Nobody in the arena
    Bi = 1  # In the back - coming in
    Fi = 2  # In the front - coming in
    Bo = 3  # In the back - coming out
    Fo = 4  # In the front - coming out

def _get_placement(targets):
    if len(targets) is 0:
        return Placement.Empty
    if targets[0].yPosCm > 0:
        return Placement.Front
    if targets[0].yPosCm <= 0:
        return Placement.Back
  
	#this is my clasified information! 
class PersonCounter(WalabotHandler):
    connection_id = 'WalabotCloudMQTT1'
    broker_ip_address = "m12.cloudmqtt.com"
    broker_port = 15238
    broker_username = 'qujgftta'
    broker_password = 'qTe7J7W6dimk'
    
    def __init__(self): 
        super(PersonCounter, self).__init__()
        self._is_connected = False
        GPIO.output(19,GPIO.LOW)
        
        self.placement = Placement.Empty
        self.state = State.Idle
        self.count = 0
        self.state_machine = {
            State.Idle:
                {Placement.Empty: State.Idle,
                 Placement.Back: State.Bi,
                 Placement.Front: State.Fo},
            State.Bi:
                {Placement.Empty: State.Idle,
                 Placement.Back: State.Bi,
                 Placement.Front: State.Fi},
            State.Fi:
                {Placement.Empty: State.Idle,  # increment
                 Placement.Back: State.Bi,
                 Placement.Front: State.Fi},
            State.Fo:
                {Placement.Empty: State.Idle,
                 Placement.Back: State.Bo,
                 Placement.Front: State.Fo},
            State.Bo:
                {Placement.Empty: State.Idle,  # increment
                 Placement.Back: State.Bo,
                 Placement.Front: State.Fo},
        }
    
    def update_state_get_count(self, targets):
        self.placement = _get_placement(targets)
        prev_state = self.state
        self.state = self.state_machine[self.state][self.placement]

        if prev_state == State.Bo and self.state == State.Idle:
            self._decrement()
        elif prev_state == State.Fi and self.state == State.Idle:
            self._increment()

        return self.count

    def _increment(self):
        #print "increment"
        self.count += 1
        return State.Idle

    def _decrement(self):
        #print "decrement"
        self.count = max(self.count - 1, 0)
        return State.Idle

    def start(self):
        if self._is_connected:
            return True
        try:
            # Initializes walabot lib
            self._wlbt.Initialize()
            # 1) Connect : Establish communication with walabot.
            self._wlbt.ConnectAny()
            # Set Profile - to Sensor-Narrow.
            self._wlbt.SetProfile(self._wlbt.PROF_TRACKER)
            # Dynamic-imaging filter for the specific frequencies typical of breathing
            self._wlbt.SetDynamicImageFilter(self._wlbt.FILTER_TYPE_MTI)
            # Setup arena - specify it by Cartesian coordinates.
            self._wlbt.SetArenaR(arena_params['min_in_cm'], arena_params['max_in_cm'], arena_params['res_in_cm'])
            # Sets polar range and resolution of arena (parameters in degrees).
            self._wlbt.SetArenaTheta(arena_params['min_in_degrees'], arena_params['max_in_degrees'],
                                      arena_params['res_in_degrees'])
            # Sets azimuth range and resolution of arena.(parameters in degrees).
            self._wlbt.SetArenaPhi(arena_params['min_phi_degrees'], arena_params['max_phi_degrees'],
                                    arena_params['res_phi_degrees'])
            # 3) Start: Start the system in preparation for scanning.
            self._wlbt.Start()
            self._is_connected = True
            
            return True
        except self._wlbt.WalabotError:
            traceback.print_exc()
        return False
    
    def get_data(self, data_dict):
        global num_of_people
        
        try:          
            # 4) Trigger: Scan (sense) according to profile and record signals
            # to be available for processing and retrieval.
            self._wlbt.Trigger()
            # 5) Get action: retrieve the last completed triggered recording
            targets = self._wlbt.GetTrackerTargets()
            # 6) Sort targets by amplitude
            targets = sorted(targets, key=lambda x: x.zPosCm, reverse=True)
            # 7) Update state and get people count
            prev_num_of_people = num_of_people
            #print "number of people:"
            #print num_of_people
            temp_numOfPeople = self.update_state_get_count(targets)
            num_of_people = temp_numOfPeople
            
            if prev_num_of_people != num_of_people:
                print "number of people:"
                print num_of_people
                
				# Alarm on! (Turn on LED)
                GPIO.output(19,GPIO.HIGH)
                
                nearby_devices = bluetooth.discover_devices()
                strPresence = "No"
				
				# scan for all nearby bluetooth devices (that are discoverable)
                for bdaddr in nearby_devices:
                    print str(bluetooth.lookup_name(bdaddr)) 
                    print str(bdaddr)
					
					# If one of the found devices has the word "nurse" in it's name: actiate the nurse presence functionality
                    if "nurse" in str(bluetooth.lookup_name( bdaddr )):
                        print "The Nurse: ", bluetooth.lookup_name( bdaddr )," is nearby the room!"
                        strPresence = "Yes: ##", bluetooth.lookup_name( bdaddr ),"###"
						# turn off Alarm! (since there is a nurse nearby)
                        GPIO.output(19,GPIO.LOW)
						
				# get the Raspberry Pi's IP Address
                ipaddress = commands.getoutput('hostname -I')
				# get the Walabot's Software Version
                walabotSWVersion = self._wlbt.GetVersion()
				
				# Collect all the needed data and combine it in a string to send
                data = "people:", num_of_people, "#nurse:",strPresence,"#IP:", ipaddress,"#version:",walabotSWVersion,"#"   

				# Send the string. 
                data_dict['energy'] = data

        except self._wlbt.WalabotError:
            traceback.print_exc()
            return False
        return True

    def stop(self):
        if self._is_connected:
            self._wlbt.Stop()
            self._wlbt.Disconnect()
        print('Terminate successfully')
        self._wlbt.Clean()
        self._is_connected = False

if __name__ == '__main__':
    mqtt = WalabotMQTT(PersonCounter)
